/* part_db_tr.c - simulated database access to parts 
 * uses binary tree for in-memory storage
 */
#include "tree.h"
#include "part.h"
#define TPART struct tr_part
TPART
	{
	TPART *right;
	TPART *left;
	PART part;
	};
static TPART tpart = {0};
static TPART *parts = NULL;

/* db_cmp_part - comparison function for database access */
static int db_cmp_part(p1, p2)
	TPART *p1, *p2;
	{
	return (strcmp(p1->part.part_no, p2->part.part_no));
	}
/* (more) */																															/*
.PE
.PS 16
/* db_add_part - add a part record to database */
bool db_add_part(partp)
	PART *partp;
	{
	TPART *p;

	p = (TPART *)malloc(sizeof(TPART));
	if (p == NULL)
		{
		remark("Part storage is full", "");
		return (NO);
		}
	STRUCTASST(p->part, *partp);
	TR_INSERT(&parts, p, db_cmp_part);
	return (YES);
	}
																					                                                      /* 
.PE
.PS  50
/* db_find_part - find a part record in database */
bool db_find_part(partp)
	PART *partp;
	{
	TPART *pn;		/* ptr to part record in tree */

	STRUCTASST(tpart.part, *partp);
	pn = (TPART *)TR_FIND(&parts, &tpart, db_cmp_part);
	if (pn == NULL)
		return (NO);
	STRUCTASST(*partp, pn->part);
	return (YES);
	}
/* db_del_part - delete a part record in database */
bool db_del_part(partp)
	PART *partp;
	{
	TPART **pln;		/* ptr to link-in of part record in tree */

	STRUCTASST(tpart.part, *partp);
	pln = (TPART **)TR_LFIND(&parts, &tpart, db_cmp_part);
	if (*pln == NULL)
		{
		remark("No record for this part", "");
		return (NO);
		}
	TR_DELETE(&parts, pln, db_cmp_part);
	return (YES);
	}
/* db_chg_part - change a part record in database */
bool db_chg_part(partp)
	PART *partp;
	{
	TPART *pn;		/* ptr to part record in tree */

	STRUCTASST(tpart.part, *partp);
	pn = (TPART *)TR_FIND(&parts, &tpart, db_cmp_part);
	if (pn == NULL)		/* validation done in menu should prevent */
		return (NO);	/* non-existent part from reaching here */
	STRUCTASST(pn->part, *partp);
	return (YES);
	}
/* (more) */																															/*
.PE
.PS  50
/* db_open_part - open a part record database */
void db_open_part()
	{
	TR_OPEN(&parts);
	}

/* db_close_part - close the part record database */
void db_close_part()
	{
	TR_CLOSE(&parts);
	}
